#!/usr/bin/env ruby

# dillo_proxy 1.02
# proxy for dillo 3 to be japanized
# ver 1.00: 2013/07/28 by KGK (kgk@film.rlss.okayama-u.ac.jp)
# ver 1.01: rescue for double execute. 2013/07/31 by KGK
# ver 1.02: add trap for SIGTERM. 2013/08/03 by KGK

# cf. http://thinkit.co.jp/article/117/3?page=0,0

require 'webrick'
require 'webrick/httpproxy'
require 'kconv'
include WEBrick

# cf. http://www2u.biglobe.ne.jp/~MAS/ruby/CAB8BBFAA5B3A1BCA5C9A4CEBFE4C2AC.html
CODE_NAME = {
  Kconv::JIS => 'ISO-2022-JP',
  Kconv::EUC => 'EUC-JP',
  Kconv::SJIS => 'Shift_JIS',
  Kconv::BINARY => '',
  Kconv::UNKNOWN => '',
  Kconv::ASCII => '',
  Kconv::UTF8 => 'UTF-8',
  Kconv::UTF16 => 'UTF-16',
}

class DilloHTTPProxyServer < HTTPProxyServer
  def proxy_service(req, res)
    if %r|www\.google\.co\.jp$|.match(req.host)
      uri_s = res.request_uri.to_s
      if %r/search/.match(uri_s) and ! %r/&oe=UTF-8/.match(uri_s)
        if uri_s.gsub!(/&oe=[^&]+/, '&oe=UTF-8')
          uri = URI.parse(uri_s)
        else
          uri = URI.parse(uri_s + '&oe=UTF-8')
        end
#        print ">redirect: #{uri_s} -> #{uri.to_s}\n"
        self.logger.debug(">redirect: #{uri_s} -> #{uri.to_s}")
        res.set_redirect(WEBrick::HTTPStatus::TemporaryRedirect, uri)
        return
      end
    end
    super
    self.logger.debug(">request uri: #{res.request_uri.to_s}")
    if res.header['content-type'] =~ %r!^text/html!
      guess = CODE_NAME[Kconv.guess(res.body)]
      if guess != '' and ! %r/#{guess}/.match(res.header["content-type"])
        ct = "text/html; charset=#{guess}"
#        print ">Content-Type: #{res.header["content-type"]} -> #{ct}\n"
        self.logger.debug(">Content-Type: #{res.header["content-type"]} -> #{ct}")
        res.header["content-type"] = ct
      end
      res.body.sub!(/(<META .*; Charset=)x-sjis/i, '\1Shift_JIS')
    end
  end
end

alog="#{ENV['HOME']}/log/dillo_proxy.log"
dlog="#{ENV['HOME']}/log/dillo_proxy_debug.log"
config = {:Port => 8080,
  :Logger => WEBrick::Log::new(dlog, WEBrick::Log::DEBUG), 
  :AccessLog => [[File.open(alog, 'w'), WEBrick::AccessLog::CLF]]}

s = DilloHTTPProxyServer.new(config) rescue exit(0)

File.delete(alog) if File.exist?(alog)
File.delete(dlog) if File.exist?(dlog)
trap("INT"){s.shutdown}
trap("TERM"){s.shutdown}
s.start

# end of script
